import hashlib
import logging
import re
import time

from streamlink.exceptions import PluginError
from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream
from streamlink.utils import parse_json

log = logging.getLogger(__name__)


class Huomao(Plugin):
    magic_val = '6FE26D855E1AEAE090E243EB1AF73685'
    mobile_url = 'https://m.huomao.com/mobile/mob_live/{0}'
    live_data_url = 'https://m.huomao.com/swf/live_data'
    vod_url = 'https://www.huomao.com/video/vreplay/{0}'

    author = None
    category = None
    title = None

    url_re = re.compile(r'''
        (?:https?://)?(?:www\.)?huomao(?:\.tv|\.com)
        (?P<path>/|/video/v/)
        (?P<room_id>\d+)
    ''', re.VERBOSE)

    author_re = re.compile(
        r'<p class="nickname_live">\s*<span>\s*(.*?)\s*</span>',
        re.DOTALL,
    )

    title_re = re.compile(
        r'<p class="title-name">\s*(.*?)\s*</p>',
        re.DOTALL,
    )

    video_id_re = re.compile(r'var stream = "([^"]+)"')
    video_res_re = re.compile(r'_([\d]+p?)\.m3u8')
    vod_data_re = re.compile(r'var video = ({.*});')

    _live_data_schema = validate.Schema({
        'roomStatus': validate.transform(lambda x: int(x)),
        'streamList': [{'list_hls': [{
            'url': validate.url(),
        }]}],
    })

    _vod_data_schema = validate.Schema({
        'title': validate.text,
        'username': validate.text,
        'vaddress': validate.all(
            validate.text,
            validate.transform(parse_json),
            [{
                'url': validate.url(),
                'vheight': int,
            }],
        ),
    })

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def _get_live_streams_data(self, video_id):
        client_type = 'huomaomobileh5'
        time_now = str(int(time.time()))

        token_data = "{0}{1}{2}{3}".format(
            video_id,
            client_type,
            time_now,
            self.magic_val,
        )

        token = hashlib.md5(bytes(token_data, 'utf-8')).hexdigest()
        log.debug("Token={0}".format(token))

        post_data = {
            'cdns': 1,
            'streamtype': 'live',
            'VideoIDS': video_id,
            'from': client_type,
            'time': time_now,
            'token': token,
        }
        video_data = self.session.http.post(self.live_data_url, data=post_data)

        return self.session.http.json(
            video_data,
            schema=self._live_data_schema,
        )

    def _get_vod_streams(self, vod_id):
        res = self.session.http.get(self.vod_url.format(vod_id))
        m = self.vod_data_re.search(res.text)
        vod_json = m and m.group(1)

        if vod_json is None:
            raise PluginError("Failed to get VOD data")

        vod_data = parse_json(vod_json, schema=self._vod_data_schema)

        self.author = vod_data['username']
        self.category = 'VOD'
        self.title = vod_data['title']

        vod_data = vod_data['vaddress']

        streams = {}
        for stream in vod_data:
            video_res = stream['vheight']

            if 'p' not in str(video_res):
                video_res = "{0}p".format(video_res)

            if video_res in streams:
                video_res = "{0}_alt".format(video_res)

            streams[video_res] = HLSStream(self.session, stream['url'])

        return streams

    def _get_live_streams(self, room_id):
        res = self.session.http.get(self.mobile_url.format(room_id))

        m = self.author_re.search(res.text)
        if m:
            self.author = m.group(1)

        self.category = 'Live'

        m = self.title_re.search(res.text)
        if m:
            self.title = m.group(1)

        m = self.video_id_re.search(res.text)
        video_id = m and m.group(1)

        if video_id is None:
            raise PluginError("Failed to get video ID")
        else:
            log.debug("Video ID={0}".format(video_id))

        streams_data = self._get_live_streams_data(video_id)

        if streams_data['roomStatus'] == 0:
            log.info("This room is currently inactive: {0}".format(room_id))
            return

        streams_data = streams_data['streamList'][0]['list_hls']

        streams = {}
        for stream in streams_data:
            m = self.video_res_re.search(stream['url'])
            video_res = m and m.group(1)
            if video_res is None:
                continue

            if 'p' not in video_res:
                video_res = "{0}p".format(video_res)

            if video_res in streams:
                video_res = "{0}_alt".format(video_res)

            streams[video_res] = HLSStream(self.session, stream['url'])

        return streams

    def get_author(self):
        if self.author is not None:
            return self.author

    def get_category(self):
        if self.category is not None:
            return self.category

    def get_title(self):
        if self.title is not None:
            return self.title

    def _get_streams(self):
        path, url_id = self.url_re.search(self.url).groups()
        log.debug("Path={0}".format(path))
        log.debug("URL ID={0}".format(url_id))

        if path != '/':
            return self._get_vod_streams(url_id)
        else:
            return self._get_live_streams(url_id)


__plugin__ = Huomao



import logging
import re

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream
from streamlink.utils import parse_json

log = logging.getLogger(__name__)


class Vlive(Plugin):
    _url_re = re.compile(r"https://www\.vlive\.tv/(?P<format>video|post)/(?P<id>[0-9\-]+)")
    _page_info = re.compile(r'window\.__PRELOADED_STATE__\s*=\s*({.*})\s*<', re.DOTALL)
    _playinfo_url = "https://www.vlive.tv/globalv-web/vam-web/old/v3/live/{0}/playInfo"

    _schema_video = validate.Schema(
        validate.transform(_page_info.search),
        validate.any(None, validate.all(
            validate.get(1),
            validate.transform(parse_json),
            validate.any(validate.all(
                {"postDetail": {"post": {"officialVideo": {
                    "type": str,
                    "videoSeq": int,
                    validate.optional("status"): str,
                }}}},
                validate.get("postDetail"),
                validate.get("post"),
                validate.get("officialVideo")),
                validate.all(
                    {"postDetail": {"error": {
                        "errorCode": str,
                    }}},
                    validate.get("postDetail"),
                    validate.get("error")))
        ))
    )

    _schema_stream = validate.Schema(
        validate.transform(parse_json),
        validate.all(
            {"result": {"streamList": [{
                "streamName": str,
                "serviceUrl": str,
            }]}},
            validate.get("result"),
            validate.get("streamList")
        )
    )

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url) is not None

    def _get_streams(self):
        video_json = self.session.http.get(self.url, headers={"Referer": self.url},
                                           schema=self._schema_video)
        if video_json is None:
            log.error('Could not parse video page')
            return

        err = video_json.get('errorCode')
        if err == 'common_700':
            log.error('Available only to members of the channel')
            return
        elif err == 'common_702':
            log.error('Vlive+ VODs are not supported')
            return
        elif err == 'common_404':
            log.error('Could not find video page')
            return
        elif err is not None:
            log.error('Unknown error code: {0}'.format(err))
            return

        if video_json['type'] == 'VOD':
            log.error('VODs are not supported')
            return

        url_format, video_id = self._url_re.match(self.url).groups()
        if url_format == 'post':
            video_id = str(video_json['videoSeq'])

        video_status = video_json.get('status')
        if video_status == 'ENDED':
            log.error('Stream has ended')
            return
        elif video_status != 'ON_AIR':
            log.error('Unknown video status: {0}'.format(video_status))
            return

        stream_info = self.session.http.get(self._playinfo_url.format(video_id),
                                            headers={"Referer": self.url},
                                            schema=self._schema_stream)

        # All "resolutions" have a variant playlist with only one entry, so just combine them
        for i in stream_info:
            res_streams = HLSStream.parse_variant_playlist(self.session, i['serviceUrl'])
            if len(res_streams.values()) > 1:
                log.warning('More than one stream in variant playlist, using first entry!')

            yield i['streamName'], res_streams.popitem()[1]


__plugin__ = Vlive



import base64
import logging
import random
import re
import time
from html import unescape as html_unescape
from urllib.parse import urlparse

from streamlink.plugin import Plugin, PluginArgument, PluginArguments
from streamlink.plugin.api import validate
from streamlink.plugin.api.utils import itertags
from streamlink.stream import HLSStream
from streamlink.utils import parse_qsd
from streamlink.utils.crypto import decrypt_openssl

log = logging.getLogger(__name__)


class Streann(Plugin):
    arguments = PluginArguments(
        PluginArgument(
            "url",
            type=str,
            metavar="URL",
            help="""
            Source URL where the iframe is located,
            only required for direct URLs of `ott.streann.com`
            """
        )
    )

    _url_re = re.compile(r"""(?x)https?://(?:
        ott\.streann\.com/s(?:treaming|-secure)/player\.html
        |
        (?:www\.)?(?:
            centroecuador\.ec
            |
            columnaestilos\.com
            |
            crc.cr/estaciones/
            |
            evtv\.online/noticias-de-venezuela/
            |
            telecuracao\.com
            |
            willax\.tv/en-vivo
        )
    )""")

    base_url = "https://ott.streann.com"
    get_time_url = base_url + "/web/services/public/get-server-time"
    token_url = base_url + "/loadbalancer/services/web-players/{playerId}/token/{type}/{dataId}/{deviceId}"
    stream_url = base_url + "/loadbalancer/services/web-players/{type}s-reseller-secure/{dataId}/{playerId}" \
                            "/{token}/{resellerId}/playlist.m3u8?date={time}&device-type=web&device-name=web" \
                            "&device-os=web&device-id={deviceId}"
    passphrase_re = re.compile(r'''CryptoJS\.AES\.decrypt\(.*?,\s*(['"])(?P<passphrase>(?:(?!\1).)*)\1\s*?\);''')

    _device_id = None
    _domain = None
    title = None

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url) is not None

    def get_title(self):
        return self.title

    @property
    def device_id(self):
        """
        Randomly generated deviceId.
        :return:
        """
        if self._device_id is None:
            self._device_id = "".join(
                random.choice("0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ") for _ in range(50))
        return self._device_id

    @property
    def time(self):
        res = self.session.http.get(self.get_time_url)
        data = self.session.http.json(res)
        return str(data.get("serverTime", int(time.time() * 1000)))

    def passphrase(self):
        log.debug("passphrase")
        res = self.session.http.get(self.url)
        passphrase_m = self.passphrase_re.search(res.text)
        return passphrase_m and passphrase_m.group("passphrase").encode("utf8")

    def get_token(self, **config):
        log.debug("get_token")
        pdata = dict(arg1=base64.b64encode(self._domain.encode("utf8")),
                     arg2=base64.b64encode(self.time.encode("utf8")))

        headers = {
            "Referer": self.url,
            "X-Requested-With": "XMLHttpRequest",
            "Content-Type": "application/x-www-form-urlencoded"
        }

        res = self.session.http.post(
            self.token_url.format(deviceId=self.device_id, **config),
            data=pdata,
            headers=headers
        )

        if res.status_code == 204:
            log.error(f"self._domain might be invalid - {self._domain}")
            return

        data = self.session.http.json(res, schema=validate.Schema({
            "token": str,
            validate.optional("name"): str,
            validate.optional("webPlayer"): {
                validate.optional("id"): str,
                validate.optional("name"): str,
                validate.optional("type"): str,
                validate.optional("allowedDomains"): [str],
            },
        }))
        log.trace(f"{data!r}")
        self.title = data.get("name")
        return data["token"]

    def _get_streams(self):
        p = urlparse(self.url)
        if "ott.streann.com" != p.netloc:
            self._domain = p.netloc
            res = self.session.http.get(self.url)
            for iframe in itertags(res.text, "iframe"):
                iframe_url = html_unescape(iframe.attributes.get("src"))
                if "ott.streann.com" in iframe_url:
                    self.url = iframe_url
                    break
            else:
                log.error("Could not find 'ott.streann.com' iframe")
                return

        if not self._domain and self.get_option("url"):
            self._domain = urlparse(self.get_option("url")).netloc

        if self._domain is None:
            log.error("Missing source URL use --streann-url")
            return

        self.session.http.headers.update({"Referer": self.url})
        # Get the query string
        encrypted_data = urlparse(self.url).query
        data = base64.b64decode(encrypted_data)
        # and decrypt it
        passphrase = self.passphrase()
        if passphrase:
            log.debug("Found passphrase")
            params = decrypt_openssl(data, passphrase)
            config = parse_qsd(params.decode("utf8"))
            log.trace(f"config: {config!r}")
            token = self.get_token(**config)
            if not token:
                return
            hls_url = self.stream_url.format(time=self.time,
                                             deviceId=self.device_id,
                                             token=token,
                                             **config)
            log.debug("URL={0}".format(hls_url))
            return HLSStream.parse_variant_playlist(self.session,
                                                    hls_url,
                                                    acceptable_status=(200, 403, 404, 500))


__plugin__ = Streann



"""Plugin for Arte.tv, bi-lingual art and culture channel."""

import logging
import re
from operator import itemgetter

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream

log = logging.getLogger(__name__)
JSON_VOD_URL = "https://api.arte.tv/api/player/v1/config/{0}/{1}?platform=ARTE_NEXT"
JSON_LIVE_URL = "https://api.arte.tv/api/player/v1/livestream/{0}"

_url_re = re.compile(r"""
    https?://(?:\w+\.)?arte\.tv/(?:guide/)?
    (?P<language>[a-z]{2})/
    (?:
        (?:videos/)?(?P<video_id>(?!RC\-|videos)[^/]+?)/.+ | # VOD
        (?:direct|live)        # Live TV
    )
""", re.VERBOSE)

_video_schema = validate.Schema({
    "videoJsonPlayer": {
        "VSR": validate.any(
            [],
            {
                validate.text: {
                    "height": int,
                    "mediaType": validate.text,
                    "url": validate.text,
                    "versionProg": int,
                    "versionLibelle": validate.text
                },
            },
        )
    }
})


class ArteTV(Plugin):
    @classmethod
    def can_handle_url(cls, url):
        return _url_re.match(url)

    def _create_stream(self, streams):
        variant, variantname = min([(stream["versionProg"], stream["versionLibelle"]) for stream in streams.values()],
                                   key=itemgetter(0))
        log.debug(f"Using the '{variantname}' stream variant")
        for sname, stream in streams.items():
            if stream["versionProg"] == variant:
                if stream["mediaType"] == "hls":
                    try:
                        streams = HLSStream.parse_variant_playlist(self.session, stream["url"])
                        yield from streams.items()
                    except OSError as err:
                        log.warning(f"Failed to extract HLS streams for {sname}/{stream['versionLibelle']}: {err}")

    def _get_streams(self):
        match = _url_re.match(self.url)
        language = match.group('language')
        video_id = match.group('video_id')
        if video_id is None:
            json_url = JSON_LIVE_URL.format(language)
        else:
            json_url = JSON_VOD_URL.format(language, video_id)
        res = self.session.http.get(json_url)
        video = self.session.http.json(res, schema=_video_schema)

        if not video["videoJsonPlayer"]["VSR"]:
            return

        vsr = video["videoJsonPlayer"]["VSR"]
        return self._create_stream(vsr)


__plugin__ = ArteTV



import logging
import re
from urllib.parse import urlparse, urlunparse

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream
from streamlink.utils import parse_json

log = logging.getLogger(__name__)


class RaiPlay(Plugin):
    _re_url = re.compile(r"https?://(?:www\.)?raiplay\.it/dirette/(\w+)/?")

    _re_data = re.compile(r"data-video-json\s*=\s*\"([^\"]+)\"")
    _schema_data = validate.Schema(
        validate.transform(_re_data.search),
        validate.any(None, validate.get(1))
    )
    _schema_json = validate.Schema(
        validate.transform(parse_json),
        validate.get("video"),
        validate.get("content_url"),
        validate.url()
    )

    @classmethod
    def can_handle_url(cls, url):
        return cls._re_url.match(url) is not None

    def _get_streams(self):
        json_url = self.session.http.get(self.url, schema=self._schema_data)
        if not json_url:
            return

        json_url = urlunparse(urlparse(self.url)._replace(path=json_url))
        log.debug("Found JSON URL: {0}".format(json_url))

        stream_url = self.session.http.get(json_url, schema=self._schema_json)
        log.debug("Found stream URL: {0}".format(stream_url))

        res = self.session.http.request("HEAD", stream_url)
        # status code will be 200 even if geo-blocked, so check the returned content-type
        if not res or not res.headers or res.headers["Content-Type"] == "video/mp4":
            log.error("Geo-restricted content")
            return

        yield from HLSStream.parse_variant_playlist(self.session, stream_url).items()


__plugin__ = RaiPlay



import logging
import re
from html import unescape as html_unescape
from urllib.parse import urlparse

from streamlink.plugin import Plugin
from streamlink.plugin.api import useragents, validate
from streamlink.stream import HLSStream
from streamlink.utils import update_scheme

log = logging.getLogger(__name__)


class CDNBG(Plugin):
    url_re = re.compile(r"""
        https?://(?:www\.)?(?:
            armymedia\.bg|
            bgonair\.bg/tvonline|
            bloombergtv\.bg/video|
            (?:tv\.)?bnt\.bg/\w+(?:/\w+)?|
            live\.bstv\.bg|
            i\.cdn\.bg/live/|
            nova\.bg/live|
            mu-vi\.tv/LiveStreams/pages/Live\.aspx
        )/?
    """, re.VERBOSE)
    iframe_re = re.compile(r"iframe .*?src=\"((?:https?(?::|&#58;))?//(?:\w+\.)?cdn.bg/live[^\"]+)\"", re.DOTALL)
    sdata_re = re.compile(r"sdata\.src.*?=.*?(?P<q>[\"'])(?P<url>http.*?)(?P=q)")
    hls_file_re = re.compile(r"(src|file): (?P<q>[\"'])(?P<url>(https?:)?//.+?m3u8.*?)(?P=q)")
    hls_src_re = re.compile(r"video src=(?P<url>http[^ ]+m3u8[^ ]*)")

    stream_schema = validate.Schema(
        validate.any(
            validate.all(validate.transform(sdata_re.search), validate.get("url")),
            validate.all(validate.transform(hls_file_re.search), validate.get("url")),
            validate.all(validate.transform(hls_src_re.search), validate.get("url")),
        )
    )

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def find_iframe(self, url):
        self.session.http.headers.update({"User-Agent": useragents.CHROME})
        res = self.session.http.get(self.url)
        for iframe_url in self.iframe_re.findall(res.text):
            if "googletagmanager" not in iframe_url:
                iframe_url = html_unescape(iframe_url)
                return update_scheme(self.url, iframe_url)

    def _get_streams(self):
        if "cdn.bg" in urlparse(self.url).netloc:
            iframe_url = self.url
        else:
            iframe_url = self.find_iframe(self.url)

        log.debug(f"Found iframe: {iframe_url}")
        res = self.session.http.get(iframe_url, headers={"Referer": self.url})
        stream_url = update_scheme(self.url, self.stream_schema.validate(res.text))
        log.warning("SSL Verification disabled.")
        return HLSStream.parse_variant_playlist(self.session,
                                                stream_url,
                                                verify=False)


__plugin__ = CDNBG



import logging
import re
from urllib.parse import urljoin

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream

log = logging.getLogger(__name__)


class Dogan(Plugin):
    """
    Support for the live streams from Doğan Media Group channels
    """
    url_re = re.compile(r"""
        https?://(?:www\.)?
        (?:cnnturk\.com/(?:action/embedvideo/.*|canli-yayin|tv-cnn-turk|video/.*)|
           dreamturk\.com\.tr/(?:canli|canli-yayin-izle|dream-turk-ozel/.*|programlar/.*)|
           dreamtv\.com\.tr/dream-ozel/.*|
           kanald\.com\.tr/.*|
           teve2\.com\.tr/(?:canli-yayin|diziler/.*|embed/.*|filmler/.*|programlar/.*))
    """, re.VERBOSE)
    playerctrl_re = re.compile(r'''<div\s+id="video-element".*?>''', re.DOTALL)
    data_id_re = re.compile(r'''data-id=(?P<quote>["'])/?(?P<id>\w+)(?P=quote)''')
    content_id_re = re.compile(r'"content[Ii]d",\s*"(\w+)"')
    item_id_re = re.compile(r"_itemId\s+=\s+'(\w+)';")
    content_api = "/actions/media?id={id}"
    dream_api = "/actions/content/media/{id}"
    new_content_api = "/action/media/{id}"
    content_api_schema = validate.Schema(
        {
            "data": {
                "id": str,
                "media": {
                    "link": {
                        validate.optional("defaultServiceUrl"): validate.any(validate.url(), ""),
                        validate.optional("serviceUrl"): validate.any(validate.url(), ""),
                        "securePath": str,
                    },
                },
            },
        },
        validate.get("data"),
        validate.get("media"),
        validate.get("link"),
    )
    new_content_api_schema = validate.Schema(
        {
            "Media": {
                "Link": {
                    "ContentId": str,
                    validate.optional("DefaultServiceUrl"): validate.any(validate.url(), ""),
                    validate.optional("ServiceUrl"): validate.any(validate.url(), ""),
                    "SecurePath": str,
                },
            },
        },
        validate.get("Media"),
        validate.get("Link"),
    )

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def _get_content_id(self):
        res = self.session.http.get(self.url)
        # find the contentId
        content_id_m = self.content_id_re.search(res.text)
        if content_id_m:
            log.debug("Found contentId by contentId regex")
            return content_id_m.group(1)

        # find the PlayerCtrl div
        player_ctrl_m = self.playerctrl_re.search(res.text)
        if player_ctrl_m:
            # extract the content id from the player control data
            player_ctrl_div = player_ctrl_m.group(0)
            content_id_m = self.data_id_re.search(player_ctrl_div)
            if content_id_m:
                log.debug("Found contentId by player data-id regex")
                return content_id_m.group("id")

        # find the itemId var
        item_id_m = self.item_id_re.search(res.text)
        if item_id_m:
            log.debug("Found contentId by itemId regex")
            return item_id_m.group(1)

    def _get_new_content_hls_url(self, content_id, api_url):
        log.debug("Using new content API url")
        d = self.session.http.get(urljoin(self.url, api_url.format(id=content_id)))
        d = self.session.http.json(d, schema=self.new_content_api_schema)

        if d["DefaultServiceUrl"] == "https://www.kanald.com.tr":
            self.url = d["DefaultServiceUrl"]
            return self._get_content_hls_url(content_id)
        else:
            if d["SecurePath"].startswith("http"):
                return d["SecurePath"]
            else:
                return urljoin((d["ServiceUrl"] or d["DefaultServiceUrl"]), d["SecurePath"])

    def _get_content_hls_url(self, content_id):
        d = self.session.http.get(urljoin(self.url, self.content_api.format(id=content_id)))
        d = self.session.http.json(d, schema=self.content_api_schema)

        return urljoin((d["serviceUrl"] or d["defaultServiceUrl"]), d["securePath"])

    def _get_hls_url(self, content_id):
        # make the api url relative to the current domain
        if "cnnturk.com" in self.url or "teve2.com.tr" in self.url:
            return self._get_new_content_hls_url(content_id, self.new_content_api)
        elif "dreamturk.com.tr" in self.url or "dreamtv.com.tr" in self.url:
            return self._get_new_content_hls_url(content_id, self.dream_api)
        else:
            return self._get_content_hls_url(content_id)

    def _get_streams(self):
        content_id = self._get_content_id()
        if content_id:
            log.debug(f"Loading content: {content_id}")
            hls_url = self._get_hls_url(content_id)
            return HLSStream.parse_variant_playlist(self.session, hls_url)
        else:
            log.error("Could not find the contentId for this stream")


__plugin__ = Dogan



import json
import logging
import re
from urllib.parse import unquote_plus

from streamlink.plugin import Plugin, PluginError
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream

log = logging.getLogger(__name__)


QUALITY_WEIGHTS = {
    "src": 1080,
}


class DLive(Plugin):
    _re_url = re.compile(r"""
        https?://(?:www\.)?dlive\.tv/
        (?:
            (?:p/(?P<video>[^/]+))
            |
            (?P<channel>[^/]+)
        )
    """, re.VERBOSE)
    _re_videoPlaybackUrl = re.compile(r'"playbackUrl"\s*:\s*"([^"]+\.m3u8)"')

    _schema_userByDisplayName = validate.Schema({
        "data": {
            "userByDisplayName": {
                "livestream": validate.any(None, {
                    "title": validate.text
                }),
                "username": validate.text
            }
        }},
        validate.get("data"),
        validate.get("userByDisplayName")
    )
    _schema_videoPlaybackUrl = validate.Schema(
        validate.transform(_re_videoPlaybackUrl.search),
        validate.any(None, validate.all(
            validate.get(1),
            validate.transform(unquote_plus),
            validate.transform(lambda url: bytes(url, "utf-8").decode("unicode_escape")),
            validate.url()
        ))
    )

    @classmethod
    def can_handle_url(cls, url):
        return cls._re_url.match(url)

    @classmethod
    def stream_weight(cls, key):
        weight = QUALITY_WEIGHTS.get(key)
        if weight:
            return weight, "dlive"

        return Plugin.stream_weight(key)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.author = None
        self.title = None

        match = self._re_url.match(self.url)
        self.video = match.group("video")
        self.channel = match.group("channel")

    def get_author(self):
        return self.author

    def get_title(self):
        return self.title

    def _get_streams_video(self):
        log.debug("Getting video HLS streams for {0}".format(self.video))
        try:
            hls_url = self.session.http.get(self.url, schema=self._schema_videoPlaybackUrl)
            if hls_url is None:
                return
        except PluginError:
            return

        return HLSStream.parse_variant_playlist(self.session, hls_url)

    def _get_streams_live(self):
        log.debug("Getting live HLS streams for {0}".format(self.channel))
        try:
            data = json.dumps({"query": """query {{
                userByDisplayName(displayname:"{displayname}") {{
                    livestream {{
                        title
                    }}
                    username
                }}
            }}""".format(displayname=self.channel)})
            res = self.session.http.post("https://graphigo.prd.dlive.tv/", data=data)
            res = self.session.http.json(res, schema=self._schema_userByDisplayName)
            if res["livestream"] is None:
                return
        except PluginError:
            return

        self.author = self.channel
        self.title = res["livestream"]["title"]

        hls_url = "https://live.prd.dlive.tv/hls/live/{0}.m3u8".format(res["username"])

        return HLSStream.parse_variant_playlist(self.session, hls_url)

    def _get_streams(self):
        if self.video:
            return self._get_streams_video()
        elif self.channel:
            return self._get_streams_live()


__plugin__ = DLive



import logging
import random
import re
from io import BytesIO
from urllib.parse import parse_qsl, urlparse

from streamlink import PluginError
from streamlink.packages.flashmedia import AMFMessage, AMFPacket
from streamlink.packages.flashmedia.types import AMF3ObjectBase
from streamlink.plugin import Plugin
from streamlink.plugin.api import useragents, validate
from streamlink.stream import HLSStream, HTTPStream, RTMPStream

log = logging.getLogger(__name__)


@AMF3ObjectBase.register("com.brightcove.experience.ViewerExperienceRequest")
class ViewerExperienceRequest(AMF3ObjectBase):
    def __init__(self, experienceId, URL, playerKey, deliveryType, TTLToken, contentOverrides):
        self.experienceId = experienceId
        self.URL = URL
        self.playerKey = playerKey
        self.deliveryType = deliveryType
        self.TTLToken = TTLToken
        self.contentOverrides = contentOverrides


@AMF3ObjectBase.register("com.brightcove.experience.ContentOverride")
class ContentOverride(AMF3ObjectBase):
    def __init__(self, featuredRefId, contentRefIds, contentId, target="videoPlayer", contentIds=None,
                 contentType=0, featuredId=float('nan'), contentRefId=None):
        self.featuredRefId = featuredRefId
        self.contentRefIds = contentRefIds
        self.contentId = contentId
        self.target = target
        self.contentIds = contentIds
        self.contentType = contentType
        self.featuredId = featuredId
        self.contentRefId = contentRefId


class BrightcovePlayer:
    player_page = "http://players.brightcove.net/{account_id}/{player_id}/index.html"
    api_url = "https://edge.api.brightcove.com/playback/v1/"
    amf_broker = "http://c.brightcove.com/services/messagebroker/amf"

    policy_key_re = re.compile(r'''policyKey\s*:\s*(?P<q>['"])(?P<key>.*?)(?P=q)''')

    schema = validate.Schema({
        "sources": [{
            validate.optional("height"): validate.any(int, None),
            validate.optional("avg_bitrate"): validate.any(int, None),
            validate.optional("src"): validate.url(),
            validate.optional("app_name"): validate.any(
                validate.url(scheme="rtmp"),
                validate.url(scheme="rtmpe")
            ),
            validate.optional("stream_name"): validate.text,
            validate.optional("type"): validate.text
        }]
    })

    def __init__(self, session, account_id, player_id="default_default"):
        self.session = session
        log.debug("Creating player for account {0} (player_id={1})".format(account_id, player_id))
        self.account_id = account_id
        self.player_id = player_id

    def player_url(self, video_id):
        return self.player_page.format(account_id=self.account_id,
                                       player_id=self.player_id,
                                       params=dict(videoId=video_id))

    def video_info(self, video_id, policy_key):
        url = "{base}accounts/{account_id}/videos/{video_id}".format(base=self.api_url,
                                                                     account_id=self.account_id,
                                                                     video_id=video_id)
        res = self.session.http.get(
            url,
            headers={
                "User-Agent": useragents.CHROME,
                "Referer": self.player_url(video_id),
                "Accept": "application/json;pk={0}".format(policy_key)
            }
        )
        return self.session.http.json(res, schema=self.schema)

    def policy_key(self, video_id):
        # Get the embedded player page
        res = self.session.http.get(self.player_url(video_id))

        policy_key_m = self.policy_key_re.search(res.text)
        policy_key = policy_key_m and policy_key_m.group("key")
        if not policy_key:
            raise PluginError("Could not find Brightcove policy key")

        return policy_key

    def get_streams(self, video_id):
        log.debug("Finding streams for video: {0}".format(video_id))
        policy_key = self.policy_key(video_id)
        log.debug("Found policy key: {0}".format(policy_key))
        data = self.video_info(video_id, policy_key)
        headers = {"Referer": self.player_url(video_id)}

        for source in data.get("sources"):
            # determine quality name
            if source.get("height"):
                q = "{0}p".format(source.get("height"))
            elif source.get("avg_bitrate"):
                q = "{0}k".format(source.get("avg_bitrate") // 1000)
            else:
                q = "live"
            if ((source.get("type") == "application/x-mpegURL" and source.get("src"))
                    or (source.get("src") and ".m3u8" in source.get("src"))):
                yield from HLSStream.parse_variant_playlist(self.session, source.get("src"), headers=headers).items()
            elif source.get("app_name"):
                s = RTMPStream(self.session,
                               {"rtmp": source.get("app_name"),
                                "playpath": source.get("stream_name")})
                yield q, s
            elif source.get("src") and source.get("src").endswith(".mp4"):
                yield q, HTTPStream(self.session, source.get("src"), headers=headers)

    @classmethod
    def from_url(cls, session, url):
        purl = urlparse(url)
        querys = dict(parse_qsl(purl.query))

        account_id, player_id, _ = purl.path.lstrip("/").split("/", 3)
        video_id = querys.get("videoId")

        bp = cls(session, account_id=account_id, player_id=player_id)
        return bp.get_streams(video_id)

    @classmethod
    def from_player_key(cls, session, player_id, player_key, video_id, url=None):
        amf_message = AMFMessage("com.brightcove.experience.ExperienceRuntimeFacade.getDataForExperience",
                                 "/1",
                                 [
                                     ''.join(["{0:02x}".format(random.randint(0, 255)) for _ in range(20)]),  # random id
                                     ViewerExperienceRequest(experienceId=int(player_id),
                                                             URL=url or "",
                                                             playerKey=player_key,
                                                             deliveryType=float('nan'),
                                                             TTLToken="",
                                                             contentOverrides=[ContentOverride(
                                                                 featuredRefId=None,
                                                                 contentRefIds=None,
                                                                 contentId=int(video_id)
                                                             )])
                                 ])
        amf_packet = AMFPacket(version=3)
        amf_packet.messages.append(amf_message)

        res = session.http.post(
            cls.amf_broker,
            headers={"Content-Type": "application/x-amf"},
            data=amf_packet.serialize(),
            params=dict(playerKey=player_key),
            raise_for_status=False
        )
        data = AMFPacket.deserialize(BytesIO(res.content))
        result = data.messages[0].value
        bp = cls(session=session, account_id=int(result.publisherId))
        return bp.get_streams(video_id)


class Brightcove(Plugin):
    url_re = re.compile(r"https?://players\.brightcove\.net/.*?/index.html")

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def _get_streams(self):
        return BrightcovePlayer.from_url(self.session, self.url)


__plugin__ = Brightcove



import base64
import logging
import re
from html import unescape as html_unescape

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HTTPStream
from streamlink.utils import parse_json

log = logging.getLogger(__name__)


class Huya(Plugin):
    _re_url = re.compile(r'https?://(?:www\.)?huya\.com/(?P<channel>[^/]+)')
    _re_stream = re.compile(r'"stream"\s?:\s?"([^"]+)"')
    _schema_data = validate.Schema(
        {
            # 'status': int,
            # 'msg': validate.any(None, str),
            'data': [{
                'gameStreamInfoList': [{
                    'sCdnType': str,
                    'sStreamName': str,
                    'sFlvUrl': str,
                    'sFlvUrlSuffix': str,
                    'sFlvAntiCode': validate.all(str, validate.transform(lambda v: html_unescape(v))),
                    # 'sHlsUrl': str,
                    # 'sHlsUrlSuffix': str,
                    # 'sHlsAntiCode': validate.all(str, validate.transform(lambda v: html_unescape(v))),
                    validate.optional('iIsMultiStream'): int,
                    'iPCPriorityRate': int,
                }]
            }],
            # 'vMultiStreamInfo': [{
            #    'sDisplayName': str,
            #    'iBitRate': int,
            # }],
        },
        validate.get('data'),
        validate.get(0),
        validate.get('gameStreamInfoList'),
    )
    QUALITY_WEIGHTS = {}

    @classmethod
    def can_handle_url(cls, url):
        return cls._re_url.match(url) is not None

    @classmethod
    def stream_weight(cls, key):
        weight = cls.QUALITY_WEIGHTS.get(key)
        if weight:
            return weight, 'huya'

        return Plugin.stream_weight(key)

    def _get_streams(self):
        res = self.session.http.get(self.url)
        data = self._re_stream.search(res.text)

        if not data:
            return

        data = parse_json(base64.b64decode(data.group(1)), schema=self._schema_data)
        for info in data:
            log.trace(f'{info!r}')
            flv_url = f'{info["sFlvUrl"]}/{info["sStreamName"]}.{info["sFlvUrlSuffix"]}?{info["sFlvAntiCode"]}'
            name = f'source_{info["sCdnType"].lower()}'
            self.QUALITY_WEIGHTS[name] = info['iPCPriorityRate']
            yield name, HTTPStream(self.session, flv_url)

        log.debug(f'QUALITY_WEIGHTS: {self.QUALITY_WEIGHTS!r}')


__plugin__ = Huya



import logging
import re

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream
from streamlink.utils import parse_json
from streamlink.utils.url import update_scheme

log = logging.getLogger(__name__)


class Mico(Plugin):
    author = None
    category = None
    title = None

    url_re = re.compile(r'https?://(?:www\.)?micous\.com/live/\d+')
    json_data_re = re.compile(r'win._profile\s*=\s*({.*})')

    _json_data_schema = validate.Schema(
        validate.transform(json_data_re.search),
        validate.any(None, validate.all(
            validate.get(1),
            validate.transform(parse_json),
            validate.any(None, validate.all({
                'mico_id': int,
                'nickname': validate.text,
                'h5_url': validate.all(
                    validate.transform(lambda x: update_scheme('http:', x)),
                    validate.url(),
                ),
                'is_live': bool,
            })),
        )),
    )

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def get_author(self):
        if self.author is not None:
            return self.author

    def get_category(self):
        if self.category is not None:
            return self.category

    def get_title(self):
        if self.title is not None:
            return self.title

    def _get_streams(self):
        json_data = self.session.http.get(self.url, schema=self._json_data_schema)

        if not json_data:
            log.error('Failed to get JSON data')
            return

        if not json_data['is_live']:
            log.info('This stream is no longer online')
            return

        self.author = json_data['mico_id']
        self.category = 'Live'
        self.title = json_data['nickname']

        return HLSStream.parse_variant_playlist(self.session, json_data['h5_url'])


__plugin__ = Mico



import re

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.utils import parse_json


class CNEWS(Plugin):
    _url_re = re.compile(r'https?://(?:www\.)?cnews\.fr')
    _json_data_re = re.compile(r'jQuery\.extend\(Drupal\.settings, ({.*})\);')
    _dailymotion_url = 'https://www.dailymotion.com/embed/video/{}'

    _data_schema = validate.Schema(
        validate.transform(_json_data_re.search),
        validate.any(None, validate.all(
            validate.get(1),
            validate.transform(parse_json),
            {
                validate.optional('dm_player_live_dailymotion'): {
                    validate.optional('video_id'): str,
                },
                validate.optional('dm_player_node_dailymotion'): {
                    validate.optional('video_id'): str,
                },
            },
        )),
    )

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url) is not None

    def _get_streams(self):
        data = self.session.http.get(self.url, schema=self._data_schema)
        if 'dm_player_node_dailymotion' in data:
            return self.session.streams(self._dailymotion_url.format(
                data['dm_player_node_dailymotion']['video_id']))
        elif 'dm_player_live_dailymotion' in data:
            return self.session.streams(self._dailymotion_url.format(
                data['dm_player_live_dailymotion']['video_id']))


__plugin__ = CNEWS
